using Newtonsoft.Json;
using Syncfusion.ListView.XForms;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Windows.Input;
using TurnaApp.Converter;
using TurnaApp.Model;
using TurnaApp.View;
using Xamarin.Forms;

namespace TurnaApp.ViewModel
{
    public class FinancialPageViewModel : BaseViewModel
    {

        #region Properties

        private int financialPlanId;

        public int FinancialPlanId
        {
            get
            {
                return financialPlanId;
            }
            set
            {
                if (financialPlanId == value)
                {
                    return;
                }
                financialPlanId = value;
                OnPropertyChanged();
            }
        }

        private string remark;

        public string Remark
        {
            get
            {
                return remark;
            }
            set
            {
                if (remark == value)
                {
                    return;
                }
                remark = value;
                OnPropertyChanged();
            }
        }

        private string levelSign;

        public string LevelSign
        {
            get
            {
                return levelSign;
            }
            set
            {
                if (levelSign == value)
                {
                    return;
                }
                levelSign = value;
                OnPropertyChanged();
            }
        }

        private ObservableCollection<IncomeDetail> incomeList;

        public ObservableCollection<IncomeDetail> IncomeList
        {
            get
            {
                return incomeList;
            }
            set
            {
                this.incomeList = value;
                OnPropertyChanged(nameof(incomeList));
            }
        }

        private ObservableCollection<ExpenseDetail> expenseList;

        public ObservableCollection<ExpenseDetail> ExpenseList
        {
            get
            {
                return expenseList;
            }
            set
            {
                this.expenseList = value;
                OnPropertyChanged(nameof(expenseList));
            }
        }

        //public SfListView SfListExpense { get; set; }

        //public SfListView SfListIncome { get; set; }

        //public Command<SfListView> LoadedCommandIncome { get; set; }

        //public Command<SfListView> SelectionChangedCommandIncome { get; set; }

        //public Command<SfListView> LoadedCommandExpense { get; set; }

        //public Command<SfListView> SelectionChangedCommandExpense { get; set; }

        public Command InsertIncomeCommand { get; set; }

        public Command InsertExpenseCommand { get; set; }

        public Command DeleteIncomeCommand { get; set; }

        public Command UpdateIncomeCommand { get; set; }

        public Command DeleteExpenseCommand { get; set; }

        public Command UpdateExpenseCommand { get; set; }


        private Command<object> swipeCommandIncome;

        public Command<object> SwipeEndedCommandIncome
        {
            get { return swipeCommandIncome; }
            protected set { swipeCommandIncome = value; }
        }

        private Command<object> swipeCommandExpense;

        public Command<object> SwipeEndedCommandExpense
        {
            get { return swipeCommandExpense; }
            protected set { swipeCommandExpense = value; }
        }

        #endregion

        #region Constructor

        public FinancialPageViewModel(int financialPlanId)
        {
            FinancialPlanId = financialPlanId;

            //LoadedCommandIncome = new Command<SfListView>(OnListViewLoadedIncome);
            //SelectionChangedCommandIncome = new Command<SfListView>(OnSelectionChangedIncome);
            InsertIncomeCommand = new Command(InsertIncomeTapped);
            UpdateIncomeCommand = new Command(UpdateIncomeTapped);
            DeleteIncomeCommand = new Command(DeleteIncomeTapped);

            SwipeEndedCommandIncome = new Command<object>(OnSwipeEndedIncome);
            SwipeEndedCommandExpense = new Command<object>(OnSwipeEndedExpense);

            //LoadedCommandExpense = new Command<SfListView>(OnListViewLoadedExpense);
            //SelectionChangedCommandExpense = new Command<SfListView>(OnSelectionChangedExpense);
            InsertExpenseCommand = new Command(InsertExpenseTapped);
            UpdateExpenseCommand = new Command(UpdateExpenseTapped);
            DeleteExpenseCommand = new Command(DeleteExpenseTapped);

            GetIncomeDetailsOfFinancial(financialPlanId);
            GetExpenseDetailsOfFinancial(financialPlanId);
        }

        #endregion

        #region Methods

        //public void OnListViewLoadedIncome(SfListView incomeView)
        //{
        //    incomeView.ItemsSource = IncomeList;
        //}

        //private void OnSelectionChangedIncome(SfListView incomeView)
        //{

        //}

        private void InsertIncomeTapped(object obj)
        {
            App.Current.MainPage.Navigation.PushAsync(new UpsertIncome(false, financialPlanId, 0));
        }

        private void UpdateIncomeTapped(object obj)
        {
            var itemData = obj as IncomeDetail;

            App.Current.MainPage.Navigation.PushAsync(new UpsertIncome(true, 0, itemData.ID));
        }

        private async void DeleteIncomeTapped(object obj)
        {
            var itemData = obj as IncomeDetail;

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.DeleteIncomeDetail(itemData.ID);

            if (response.Status)
            {
                await Application.Current.MainPage.DisplayAlert("Bilgi", response.Message, "OK");
                IncomeList.Remove(itemData);
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
            }
        }

        private async void GetIncomeDetailsOfFinancial(int financialPlanId)
        {
            IncomeList = new ObservableCollection<IncomeDetail>();

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetIncomeDetailsOfFinancial(financialPlanId);

            if (response.Status)
            {
                var tempList = JsonConvert.DeserializeObject<ObservableCollection<IncomeDetail>>(response.Data.ToString());

                IncomeList.Clear();

                foreach (var item in tempList)
                    IncomeList.Add(item);
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
            }
        }

        //public void OnListViewLoadedExpense(SfListView expenseView)
        //{
        //    expenseView.ItemsSource = ExpenseList;
        //}

        //private void OnSelectionChangedExpense(SfListView expenseView)
        //{

        //}

        private void OnSwipeEndedIncome(object obj)
        {
            var swipeArgs = obj as SwipeEndedExtArgs;
        }

        private void OnSwipeEndedExpense(object obj)
        {
            var swipeArgs = obj as SwipeEndedExtArgs;
        }

        private void InsertExpenseTapped(object obj)
        {
            App.Current.MainPage.Navigation.PushAsync(new UpsertExpense(false, financialPlanId, 0));
        }

        private void UpdateExpenseTapped(object obj)
        {
            var itemData = obj as ExpenseDetail;

            App.Current.MainPage.Navigation.PushAsync(new UpsertExpense(true, 0, itemData.ID));
        }

        private async void DeleteExpenseTapped(object obj)
        {
            var itemData = obj as ExpenseDetail;

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.DeleteExpenseDetail(itemData.ID);

            if (response.Status)
            {
                await Application.Current.MainPage.DisplayAlert("Bilgi", response.Message, "OK");
                ExpenseList.Remove(itemData);
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
            }

        }

        private async void GetExpenseDetailsOfFinancial(int financialPlanId)
        {
            ExpenseList = new ObservableCollection<ExpenseDetail>();

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetExpenseDetailsOfFinancial(financialPlanId);

            if (response.Status)
            {
                var tempList = JsonConvert.DeserializeObject<ObservableCollection<ExpenseDetail>>(response.Data.ToString());

                ExpenseList.Clear();

                foreach (var item in tempList)
                    ExpenseList.Add(item);
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
            }
        }

        #endregion

    }
}
