﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Input;
using TurnaApp.Model;
using Xamarin.Forms;

namespace TurnaApp.ViewModel
{
    public class UpsertExpenseViewModel : BaseViewModel
    {

        #region Properties

        private ExpenseDetail expenseDetail;

        public ExpenseDetail ExpenseDetail
        {
            get
            {
                return expenseDetail;
            }
            set
            {
                if (expenseDetail == value)
                {
                    return;
                }
                expenseDetail = value;
                OnPropertyChanged();
            }
        }

        private int financialPlanId;

        public int FinancialPlanId
        {
            get
            {
                return financialPlanId;
            }
            set
            {
                if (financialPlanId == value)
                {
                    return;
                }
                financialPlanId = value;
                OnPropertyChanged();
            }
        }

        private string item;

        public string Item
        {
            get
            {
                return item;
            }
            set
            {
                if (item == value)
                {
                    return;
                }
                item = value;
                OnPropertyChanged();
            }
        }

        private decimal budgetAmount;

        public decimal BudgetAmount
        {
            get
            {
                return budgetAmount;
            }
            set
            {
                if (budgetAmount == value)
                {
                    return;
                }
                budgetAmount = value;
                OnPropertyChanged();
            }
        }

        private decimal actualAmount;

        public decimal ActualAmount
        {
            get
            {
                return actualAmount;
            }
            set
            {
                if (actualAmount == value)
                {
                    return;
                }
                actualAmount = value;
                OnPropertyChanged();
            }
        }

        private DateTime? targetPayDate;

        public DateTime? TargetPayDate
        {
            get
            {
                return targetPayDate;
            }
            set
            {
                if (targetPayDate == value)
                {
                    return;
                }
                targetPayDate = value;
                OnPropertyChanged();
            }
        }

        private DateTime? actualPayDate;

        public DateTime? ActualPayDate
        {
            get
            {
                return actualPayDate;
            }
            set
            {
                if (actualPayDate == value)
                {
                    return;
                }
                actualPayDate = value;
                OnPropertyChanged();
            }
        }

        private string notes;

        public string Notes
        {
            get
            {
                return notes;
            }
            set
            {
                if (notes == value)
                {
                    return;
                }
                notes = value;
                OnPropertyChanged();
            }
        }

        private CultureInfo cultureName;

        public CultureInfo CultureName
        {
            get { return cultureName; }
            set
            {
                cultureName = value;
                OnPropertyChanged();
            }
        }

        private ObservableCollection<ExpenseType> expenseType;

        public ObservableCollection<ExpenseType> ExpenseType
        {
            get { return expenseType; }
            set
            {
                expenseType = value;
                OnPropertyChanged("ExpenseType");
            }
        }

        private ExpenseType selectedExpenseType;

        public ExpenseType SelectedExpenseType
        {
            get { return selectedExpenseType; }
            set
            {
                selectedExpenseType = value;
                OnPropertyChanged("SelectedExpenseType");
            }
        }

        private ObservableCollection<ItemLevel> itemLevel;

        public ObservableCollection<ItemLevel> ItemLevel
        {
            get { return itemLevel; }
            set
            {
                itemLevel = value;
                OnPropertyChanged("ItemLevel");
            }
        }

        private ItemLevel selectedItemLevel;

        public ItemLevel SelectedItemLevel
        {
            get { return selectedItemLevel; }
            set
            {
                selectedItemLevel = value;
                OnPropertyChanged("SelectedItemLevel");
            }
        }

        public Command UpsertExpenseCommand { get; set; }

        #endregion

        #region Constructor

        public UpsertExpenseViewModel(bool isUpdate, int financialPlanId, int expenseDetailId)
        {
            GetExpenseTypes();
            GetItemLevels();

            ExpenseDetail = new ExpenseDetail();

            if (isUpdate)
            {
                FinancialPlanId = 0;
                ExpenseDetail.ID = expenseDetailId;
                GetExpenseDetail();
            }
            else
            {
                FinancialPlanId = financialPlanId;
                ExpenseDetail.ID = 0;
            }

            CultureName = new CultureInfo("tr-TR");

            UpsertExpenseCommand = new Command(UpsertExpenseClicked);
        }

        #endregion

        #region Methods

        private void GetItemLevels()
        {
            ItemLevel = new ObservableCollection<ItemLevel>();

            ItemLevel.Add(new ItemLevel() { ID = 1, Name = "Yüksek" });
            ItemLevel.Add(new ItemLevel() { ID = 2, Name = "Orta" });
            ItemLevel.Add(new ItemLevel() { ID = 3, Name = "Düşük" });
        }

        private async void GetExpenseTypes()
        {
            ExpenseType = new ObservableCollection<ExpenseType>();

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetExpenseTypes();

            if (response.Status)
            {
                var tempList = JsonConvert.DeserializeObject<ObservableCollection<ExpenseType>>(response.Data.ToString());

                ExpenseType.Clear();

                foreach (var item in tempList)
                    ExpenseType.Add(item);
            }
            else
            {
                // TODO
            }

        }

        private async void GetExpenseDetail()
        {
            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetExpenseDetail(ExpenseDetail.ID);

            if (response.Status)
            {
                ExpenseDetail = JsonConvert.DeserializeObject<ExpenseDetail>(response.Data.ToString());

                Item = ExpenseDetail.Item;
                FinancialPlanId = ExpenseDetail.FinancialPlan.ID;
                SelectedExpenseType = ExpenseType.Where(p => p.ID == ExpenseDetail.ExpenseType.ID).First();
                BudgetAmount = ExpenseDetail.BudgetAmount;
                ActualAmount = ExpenseDetail.ActualAmount;
                TargetPayDate = ExpenseDetail.TargetPayDate;
                ActualPayDate = ExpenseDetail.ActualPayDate;
                SelectedItemLevel = ItemLevel.Where(p => p.ID == ExpenseDetail.ItemLevel.ID).First();
                Notes = ExpenseDetail.Notes;
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Hata", response.Message + " " + response.Exception, "OK");
            }
        }

        private async void UpsertExpenseClicked(object obj)
        {
            if (SelectedExpenseType?.ID == 0 || SelectedExpenseType?.ID == null)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Gider tipi seçiniz!", "OK");
            }
            else if (Item == "" || Item == null)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Gider kaleminin adını giriniz!", "OK");
            }
            else if (BudgetAmount == 0)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Bütçelenen tutarı giriniz!", "OK");
            }
            else
            {
                try
                {
                    TurnaManager turna = new TurnaManager();

                    ExpenseDetail.Item = Item;
                    ExpenseDetail.FinancialPlan.ID = FinancialPlanId;
                    ExpenseDetail.ExpenseType.ID = (object)SelectedExpenseType == null ? 0 : SelectedExpenseType.ID;
                    ExpenseDetail.PaymentType.ID = 0;
                    ExpenseDetail.BudgetAmount = BudgetAmount;
                    ExpenseDetail.ActualAmount = ActualAmount;
                    ExpenseDetail.Currency.ID = 1;
                    ExpenseDetail.TargetPayDate = TargetPayDate ?? DateTime.Now;
                    ExpenseDetail.ActualPayDate = ActualPayDate ?? DateTime.Now;
                    ExpenseDetail.ItemLevel.ID = (object)SelectedItemLevel == null ? 0 : SelectedItemLevel.ID;
                    ExpenseDetail.Notes = Notes;
                    ExpenseDetail.IsDeleted = false;
                    ExpenseDetail.CreateTime = DateTime.Now;
                    ExpenseDetail.UpdateTime = DateTime.Now;
                    ExpenseDetail.CreatedBy.ID = App.appUser.ID;
                    ExpenseDetail.UpdatedBy.ID = App.appUser.ID;

                    TurnaResponse response;

                    if (ExpenseDetail.ID == 0)
                    {
                        response = await turna.InsertExpenseDetail(ExpenseDetail);
                    }
                    else
                    {
                        response = await turna.UpdateExpenseDetail(ExpenseDetail);
                    }

                    if (response.Status)
                    {
                        await Application.Current.MainPage.DisplayAlert("Bilgi", response.Message, "OK");
                        await Application.Current.MainPage.Navigation.PopAsync();
                    }
                    else
                    {
                        await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
                    }
                }
                catch (Exception ex)
                {
                    await Application.Current.MainPage.DisplayAlert("Hata", ex.Message, "OK");
                }
            }
        }

        #endregion

    }
}
